"""
$Id: $
"""

import unittest
 
from getpaid.report import subscriber, domain, schema
from getpaid.warehouse.interfaces import InventoryModified, InventoryOrderModified

from zope.app.intid.interfaces import IIntIds
from zope.app.testing import placelesssetup, ztapi
from getpaid.core.tests import base, generator

import session
import transaction
import sqlalchemy as rdb

def setUpReport( ):
    ztapi.provideUtility( IIntIds, generator.ProductIntId() )

class ReportTests(unittest.TestCase):
    
    def setUp(self):
        base.coreSetUp()
        setUpReport()
        schema.metadata.bind = rdb.create_engine( 'sqlite://')
        schema.metadata.create_all()

        # Set up some preconditions
        self.products = generator.ProductGenerator()
        self.orders   = generator.OrderGenerator()
        
        for i in range( 20 ):
            self.products.new()
            
    def tearDown(self):
        # Clean up after each test
        generator.ProductGenerator._products = {}
        self.order = None
        self.products = None
        placelesssetup.tearDown()
        schema.metadata.bind = None
        
    def test_orderSerialization(self):
        order = self.orders()
        subscriber.handleNewOrder( order, None )
        transaction.commit()
        s = session.Session()
        _order = s.query( domain.Order ).filter(
            domain.Order.order_zid == order.order_id ).first()
        self.assertNotEqual( _order, None )
        self.assertEqual( len( order.shopping_cart ), len( _order.items ) )

    def test_inventoryModified( self ):
        product = self.products.new()
        inventory = generator.Mock()
        inventory.stock = 10
        inventory.store_stock = 7        
        
        
        event = InventoryModified(
            inventory,
            product,
            10
            )

        _entry = subscriber.handleInventoryModified( inventory, event )
        self.assertEqual( _entry.action, u"added")
        self.assertEqual( _entry.stock, 10 )
        self.assertEqual( _entry.quantity, 10 )
        self.assertEqual( _entry.product.content_uid, product.uid )
        transaction.abort()
        
    def test_inventoryOrderModified( self ):

        order = self.orders()
        item = order.shopping_cart.values()[0]
        product = item.resolve()
        inventory = generator.Mock()
        inventory.stock = 10
        inventory.store_stock = 7        
        
        event = InventoryOrderModified(
            inventory,
            product,
            order,
            item.quantity 
            )

        # serialize the order
        _order = subscriber.handleNewOrder( order, None )

        # serialize the inventory modified event
        _entry = subscriber.handleInventoryOrderFulfilled( inventory, event )
        
        transaction.commit()

        # resurrect the detattached instances
        s = session.Session()
        s.update( _order )
        s.update( _entry )
        
        # verify serialization
        self.assertEqual( _order.order_id, _entry.order_id )
        self.assertTrue( isinstance(_entry.product_id, int ) )
        self.assertEqual( _entry.stock, inventory.stock )
        self.assertEqual( _entry.quantity, -item.quantity )
        
        
    def test_orderTransition( self ):
        order = self.orders()
        subscriber.handleNewOrder( order, None )        
        order.finance_workflow.fireTransition('create')
        order.fulfillment_workflow.fireTransition('create')
        subscriber.handleOrderTransition( order, None )
        transaction.commit()
        s = session.Session()
        _order = s.query( domain.Order ).filter(
            domain.Order.order_zid == order.order_id ).first()
        self.assertNotEqual( _order, None )

        self.assertEqual( _order.finance_status, order.finance_state )
        self.assertEqual( _order.fulfillment_status, order.fulfillment_state )


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ReportTests))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')





from zope import component
from zope.app.intid.interfaces import IIntIds
from sqlalchemy import orm

import schema

class Product( object ):
    """ a product to be purchased """

    def resolve( self ):
        return component.getUtility( IIntIds ).getObject( self.content_uid )
        
class InventoryEntry( object ):
    """ product inventory log entry """
    
## class LogEntry( object ):
##     """ order history log """

class Address( object ):
    """ an address """    
    
class LineItem( object ):
    """ an item in an order """

class Order( object ):
    """ """    

class Customer( object ):
    """ """

orm.mapper( Product, schema.products )
orm.mapper( InventoryEntry, schema.inventory,
            properties = {'product':orm.relation( Product ),
                          'order': orm.relation( Order ) } )

#orm.mapper( LogEntry, schema.order_log )
orm.mapper( Address, schema.addresses )

orm.mapper( LineItem, schema.items,
            properties={'product':orm.relation( Product ) } )

orm.mapper(
    Order, schema.orders,
    properties={
#      'log':orm.relation( LogEntry, backref='order' ),
      'items':orm.relation( LineItem, backref='order'),
      'shipping_address':orm.relation( Address, primaryjoin=(schema.orders.c.ship_id==schema.addresses.c.address_id)),
      'billing_address':orm.relation( Address, primaryjoin=(schema.orders.c.billing_id==schema.addresses.c.address_id)),
      }
    )

orm.mapper(
    Customer, schema.customers,
    properties = {
       'orders':orm.relation( Order, backref='contact_information' )
    }
    )

    



#from sqlalchemy.orm import session
from zope import component
from zope.app.intid.interfaces import IIntIds

import domain, sync, session
    
def handleInventoryModified( _inventory, event ):
    """
    when an inventory is modified, we record inventory adjustments to the database
    """
    def _( s ):
        entry = domain.InventoryEntry()
        product = _fetchSync( s, event )
        entry.product = product
        entry.quantity = event.stock_delta
        entry.action = u"added"
        entry.stock = _inventory.stock
        return entry
    
    return _interact( _ )

def handleInventoryAvailabilityModified( _inventory, event ):
    """
    when an item's availability changes, this is distinct from event which
    creates a product log entry, in that only the availability is changing
    not the actual amount on hand.
    """
    def _( s ):
        product = _fetchSync( s, event )
        sync.copyProductInventory( _inventory, product )
        return product
    
    return _interact( _ )


def handleInventoryOrderFulfilled( _inventory, event ):
    """
    when an order is fufilled, we record inventory levels of products
    items to the database. the event is generated per product.
    """
    
    def _( s ):
        entry = domain.InventoryEntry()

        # fetch the db product 
        iid = component.getUtility( IIntIds ).queryId( event.product )
        product = s.query( domain.Product ).filter(
            domain.Product.content_uid == iid ).first()

        # really, product should already exist at this point
        # for a fresh install, we do this to play nice for plugin
        # installs into legacy
        if product is None:
            product = domain.Product()
            sync.copyProduct( event.product, product, uid=iid )        
        else:
            sync.copyProductInventory( _inventory, product )
            
        order = s.query( domain.Order ).filter(
            domain.Order.order_zid == event.order.order_id ).first()
        
        entry.product = product
        entry.order = order
        entry.quantity = -event.stock_delta
        entry.stock = _inventory.stock
        entry.action = u"delivered"

        return entry
    
    return _interact( _ )

def handleOrderTransition( _order, event ):
    """
    when an order is transition, we record the state changes to the database
    """
    def _( s ):
        order = s.query( domain.Order ).filter(
            domain.Order.order_zid == _order.order_id ).first()
        if order is None:
            return
        sync.copyState( _order, order )
        return order
    return _interact( _ )
        
def handleNewOrder( _order, event ):
    """
    when a new order is created, we serialize it do the database.
    """
    def _( s ):
        order = s.query( domain.Order ).get( _order.order_id )
        if order is not None:
            order = domain.Order()
        sync.copyOrder( s, _order, order )
        return order
    return _interact( _ )

def _interact( func ):
    s = session.Session()
    value = func( s )
    if value is not None:
        s.save_or_update( value )
    return value

def _fetchSync( s, e ):
    # fetch the db product 
    iid = component.getUtility( IIntIds ).queryId( e.product )
    product = s.query( domain.Product ).filter(
        domain.Product.content_uid == iid ).first()
    
    if product is None:
        product = domain.Product()
        sync.copyProduct( e.product, product, uid=iid )
    else:
        sync.copyProductInventory( e.object, product )
    return product



"""

sync options.

 - sync synchronously

 - sync async

benefit to async syncing, is any errors in sync won't hose an order.

$Id: $
"""

from getpaid.core import interfaces
from getpaid.warehouse.interfaces import IProductInventory
from zope import component, schema

import domain

def copyAddress( source_addr, target_addr, prefix=None):
    field_names = schema.getFields( interfaces.IAddress ).keys()
    for f in field_names:
        fn = prefix and "%s_%s"%(prefix, f) or f
        value = getattr( source_addr, fn, None)
        setattr( target_addr, f, value )

def copyItem( source, target ):
    
    i = interfaces.IShippableLineItem.providedBy( source ) \
        and interfaces.IShippableLineItem \
        or interfaces.IPayableLineItem
    
    field_names = set( schema.getFields( i ).keys() )
    field_names.remove('item_id')
    
    for f in field_names:
        value = getattr( source, f, None)
        setattr( target, f, value )

    target.item_zid = source.uid
    target.product_code = source.__name__

def copyCustomer( source, target ):
    for f in schema.getFields( interfaces.IUserContactInformation ).keys():
        value = getattr( source, f )
        setattr( target, f, value )

def copyProduct( source, target, item=None, uid=None ):
    payable = interfaces.IPayable( source )
    uid = item and item.uid or uid
    assert uid, "must pass in either item or uid"

    target.content_uid = uid
    target.supplier_uid = payable.made_payable_by
    target.product_code = payable.product_code or payable.context.UID()
    target.type = "payable"
    target.price = payable.price

    # copy inventory information if available
    inventory = component.queryAdapter( source, IProductInventory )
    if inventory:
        target.pick_bin = inventory.pickbin
        target.stock = inventory.stock
        target.stock_reserve= inventory.store_stock

def copyProductInventory( source, target ):
    target.stock = source.stock
    target.stock_reserve = source.store_stock
    
def copyState( source, target ):
    # handle workflow states
    target.finance_status = source.finance_state
    target.fulfillment_status = source.fulfillment_state

def copyOrder( _session, source, target ):

    # identity info
    target.order_zid = source.order_id
    
    target.creation_date = source.creation_date
    
    # handle workflow states
    target.finance_status = source.finance_state
    target.fulfillment_status = source.fulfillment_state
    
    # handle addresses
    billing_address = domain.Address()
    copyAddress( source.billing_address, billing_address, 'bill' )

    # ifpeople doesn't like the integrity constraint, as their
    # shipping free orders sans billing in some cases
    if billing_address.first_line:
        target.billing_address = billing_address
    
    shipping_address = domain.Address()
    if source.shipping_address.ship_same_billing:
        target.shipping_address = billing_address
    else:
        copyAddress( source.shipping_address, shipping_address, 'ship')
        target.shipping_address = shipping_address

    # handle customer/contact info
    customer = domain.Customer()
    copyCustomer( source.contact_information, customer )
    target.contact_information = customer
    target.contact_information.user_id = source.user_id and unicode(source.user_id) or source.user_id
    
    # handle line items
    for _item in source.shopping_cart.values():
        item = domain.LineItem()
        copyItem( _item, item )
        target.items.append( item )
        
        if not interfaces.IPayableLineItem.providedBy( _item ):
            continue

        # try to find an existing record for this line item's product
        product = _session.query( domain.Product ).filter(
            domain.Product.content_uid == _item.uid ).first()
        
        # if we do find one, attach, continue
        if product is not None:
            item.product = product
            continue

        # else resolve item to product and serialize product
        payable = _item.resolve()
        if payable is None:
            continue
        
        product = domain.Product()
        copyProduct( payable, product, item=_item )
        _session.save( product )
        item.product = product
        
    



##############################################################################
#
# Copyright (c) 2006-2008 Kapil Thangavelu <kapil.foss@gmail.com>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
zope transaction manager integration for sqlalchemy

$Id: manager.py 299 2008-05-23 20:31:48Z kapilt $
"""

from zope import interface

import transaction
import transaction.interfaces

from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm.scoping import ScopedSession


class SessionDataManager( object ):
    """
    a data manager facade for sqlalchemy sessions participating in
    zope transactions.    
    """
    
    interface.implements( transaction.interfaces.IDataManager )

    def __init__(self, session):
        self.session = session
        self.joined = False

    def _check( self ):
        return bool( self.session.new or self.session.deleted or self.session.dirty )
    
    def abort(self, transaction):
        self.session.joined = False
        if self.session.transaction:
            self.session.transaction.rollback()
        self.session.clear()
        
    def commit(self, transaction):
        if not self.session.transaction:
            self.session.begin()
        if self.session.autoflush:
            return
        self.session.flush()

    def tpc_vote(self, transaction):
        self.session.joined = False        
        self.session.transaction.commit()
        self.session.clear()
        
    def tpc_abort(self, transaction):
        self.session.joined = False                
        self.session.transaction.rollback()
        self.session.clear()
        
    def sortKey(self):
        return "100-alchemist"
    
    def null( self, *args, **kw): pass

    tpc_finish = tpc_begin = null

    def register( self ):
        txn = transaction.get()
        txn.join( self )


class TransactionScoped( ScopedSession ):
    def __call__( self, **kwargs ):
        session = super( TransactionScoped, self).__call__( **kwargs )
        if not session.joined:
            data_manager = SessionDataManager( session )
            data_manager.register()
            session.joined = True
        if not session.transaction:
            session.begin()
        return session

def _zope_session( session_factory ):
    # session factory
    class ZopeSession( session_factory ):
        joined = False
        def __init__( self, **kwargs ):
            super( ZopeSession, self).__init__( **kwargs )
    return ZopeSession

Session = TransactionScoped( _zope_session( sessionmaker( autoflush=True,
                                                          transactional=True ) ) )
        
        



import datetime
import sqlalchemy as rdb

metadata = rdb.MetaData()
metadata.bind = rdb.create_engine('sqlite://')

def now( ):
    return datetime.datetime.now()

order_ids = rdb.Sequence('order_id_seq', metadata=metadata)

orders = rdb.Table( 
  "orders",
  metadata,
  rdb.Column("order_id",rdb.Integer(), primary_key=True ),
  rdb.Column("order_zid", rdb.Integer(), unique=True, nullable=False ),
  rdb.Column("billing_id", rdb.Integer,  rdb.ForeignKey('addresses.address_id') ),
  rdb.Column("ship_id", rdb.Integer, rdb.ForeignKey('addresses.address_id') ),
  rdb.Column("customer_id", rdb.Integer, rdb.ForeignKey('customers.customer_id'), nullable=False ),
  rdb.Column("creation_date", rdb.DateTime(timezone=True) ),
  rdb.Column("finance_status", rdb.Unicode(20) ),
  rdb.Column("fulfillment_status", rdb.Unicode(20) )
  )

## # not used  
## order_log = rdb.Table(
##   "order_log",
##   metadata,
##   rdb.Column("log_id", rdb.Integer, primary_key=True),
##   rdb.Column("order_id", rdb.Integer, rdb.ForeignKey('orders.order_id'), nullable=False ),
##   rdb.Column("changed_by", rdb.Unicode(20) ),
##   rdb.Column("change_date", rdb.DateTime( timezone=True) ),
##   rdb.Column("chage_kind", rdb.Unicode(30) ),
##   rdb.Column("comment", rdb.Unicode(90) ),  
##   rdb.Column("new_state", rdb.String(20) ),
##   rdb.Column("previous_state", rdb.String(20) ),  
##   rdb.Column("transition", rdb.String(20) )
##   )

items = rdb.Table( 
  "items",
  metadata,
  rdb.Column("item_id", rdb.Integer, primary_key=True ),
  rdb.Column("item_zid", rdb.String(40) ),
  rdb.Column("order_id",  rdb.Integer, rdb.ForeignKey('orders.order_id'), nullable=False ),
  rdb.Column("product_id", rdb.Integer, rdb.ForeignKey('products.product_id') ),
  rdb.Column("product_code", rdb.Unicode(200), nullable=False), 
  rdb.Column("name", rdb.UnicodeText, nullable=False),
  rdb.Column("description", rdb.UnicodeText, nullable=False), 
  rdb.Column("cost", rdb.Float(precision=2), nullable=False ),
  rdb.Column("quantity", rdb.Integer, nullable=False)
  )
  
products = rdb.Table( 
  "products",
  metadata,
  rdb.Column("product_id", rdb.Integer, primary_key=True ),
  rdb.Column("product_code",rdb.Unicode(200), unique=True, nullable=False ),
  rdb.Column("supplier_uid", rdb.Unicode(60) ),
  rdb.Column("content_uid", rdb.Integer, nullable=False ),  # five.intid reference
  rdb.Column("type", rdb.String(30), nullable=False ),
  rdb.Column("price", rdb.Float( precision=2), nullable=False ),
  rdb.Column("pick_bin", rdb.Unicode(35) ),
  rdb.Column("pallet", rdb.Unicode(35) ),
  rdb.Column("stock", rdb.Integer ), # actual units on hand in warehouse
  rdb.Column("stock_reserve", rdb.Integer )  # minus those that have been ordered  
)

inventory = rdb.Table(
  "product_inventory_log",
  metadata,
  rdb.Column('entry_id',   rdb.Integer, primary_key=True),
  rdb.Column('date',       rdb.DateTime, default=now ),
  rdb.Column("order_id",   rdb.Integer, rdb.ForeignKey('orders.order_id') ),
  rdb.Column("product_id", rdb.Integer, rdb.ForeignKey('products.product_id'), nullable=False),
  rdb.Column("quantity",   rdb.Integer, nullable=False ),
  rdb.Column("stock",      rdb.Integer, nullable=False ), 
  rdb.Column("action",     rdb.Unicode(15), nullable=False )
  )

customers = rdb.Table( 
  "customers",
  metadata,
  rdb.Column('customer_id', rdb.Integer, primary_key=True),
  rdb.Column('user_id', rdb.Unicode(80), nullable=True),
  rdb.Column('name', rdb.Unicode(100), nullable=True),
  rdb.Column('phone_number', rdb.Unicode(15) ),
  rdb.Column('email', rdb.Unicode(60) ),
  rdb.Column('marketing_preference', rdb.Boolean, default=False ),
  rdb.Column('email_html_format', rdb.Boolean )
  )
    
addresses = rdb.Table( 
  "addresses",
  metadata,
  rdb.Column("address_id", rdb.Integer, primary_key=True ),
  rdb.Column("first_line", rdb.Unicode(200), nullable=False ),
  rdb.Column("second_line", rdb.Unicode(200), nullable=True ),
  rdb.Column("city", rdb.Unicode(120), nullable=False ),
  rdb.Column("state", rdb.String(20), nullable=False ),
  rdb.Column("country", rdb.String(20), nullable=False ),
  rdb.Column("postal_code", rdb.String(10), nullable=False ),
  )


## # not used ...
## warehouses = rdb.Table(
##   "warehouses",
##   metadata,
##   rdb.Column("warehouse_id", rdb.Integer, primary_key=True ),
##   rdb.Column("address_id",  rdb.Integer, rdb.ForeignKey('addresses.address_id') ),
##   )

## outgoing_shipment = rdb.Table(
##   "outgoing_shipments",
##   metadata,
##   rdb.Column( "shipment_id",  rdb.Integer, primary_key=True),
##   rdb.Column( "order_id",  rdb.Integer, rdb.ForeignKey('orders.order_id') ),      
##   rdb.Column( "warehouse_id",  rdb.Integer, rdb.ForeignKey('warehouses.warehouse_id') ),
##   rdb.Column( "to_address", rdb.Integer, rdb.ForeignKey('addresses.address_id') ),
##   rdb.Column( "shipment_tracking", rdb.String(128) ),
##   )

def main( ):
    import sys    
    usage = "usage: %s database_uri"%(sys.argv[0])

    args = sys.argv[1:]
    if len(args) != 1:
        print usage
        sys.exit(1)
        
    db_uri = args[0]
    db = rdb.create_engine( db_uri , echo=True)
    
    metadata.bind = db
    metadata.drop_all( checkfirst=True )    
    metadata.create_all( checkfirst=True )
    
if __name__ == '__main__':
    main()




"""
$Id: $
"""

from zope import component
from zope.app.intid.interfaces import IIntIds
from sqlalchemy.orm import session
import sqlalchemy as rdb
import domain, schema

def product_history( product ):
    iid = component.getUtility( IIntIds ).queryId( product )
    s = session.Session()
    entries = s.query( domain.InventoryEntry ).select_from(
        schema.inventory.join( schema.products )
        ).filter( domain.Product.content_uid == iid
                  ).order_by( domain.InventoryEntry.date )
    return entries

_fulfillment_report = rdb.sql.text("""
select orders.creation_date,
       orders.order_zid,       
       item_orders.*
  from orders,
      (select order_id,
              count(*) as line_items,
              sum( items.quantity) as pieces
       from items group by order_id) as item_orders
  where orders.order_id = item_orders.order_id
    and orders.creation_date > :start_date
    and orders.creation_date < :end_date
  order by orders.creation_date desc
""")

def fulfillment_history( start_date, end_date ):
    connection = schema.metadata.bind.contextual_connect()
    results = connection.execute( _fulfillment_report,
                                  start_date = start_date,
                                  end_date = end_date )
    return list(results)

_fulfillment_report_summary = rdb.sql.text("""
select count( items ) as line_items,
       sum( quantity ) as pieces
  from items, orders
  where orders.creation_date > :start_date
    and orders.creation_date < :end_date  
""")

def fulfillment_summary( start_date, end_date ):
    connection = schema.metadata.bind.contextual_connect()
    results = connection.execute( _fulfillment_report_summary,
                                 start_date = start_date,
                                 end_date = end_date )
    return list(results)

def backorder_products( supplier_id ):
    s = session.Session()
    return s.query( domain.Product ).filter(
        domain.Product.stock_reserve < 0
        ).all() #.filter(
        #domain.Product.supplier_uid == supplier_id
        #).all()
    
    
    



"""
$Id: $
"""

from Products.PloneGetPaid.browser.base import BaseFormView
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile

from zope import interface, schema
from zope.formlib import form
from zc.table import column, table
from getpaid.report import report
from getpaid.report.i18n import _

class IReportSettings( interface.Interface ):
    start_date = schema.Date( title=_(u"Start Date") )
    end_date = schema.Date( title=_(u"End Date") )    

def orderLink( item, formatter):
    oid = item['order_zid']
    return u'<a href="@@admin-manage-order/%s/@@fulfillment">%s</a>'%( oid, oid )

class FulfillmentReport( BaseFormView ):

    form_description=_(u"Select the dates to view the fulfilled orders. Date Format is YYYY-MM-DD")
    form_fields = form.Fields(  IReportSettings )
    template = ZopeTwoPageTemplateFile('fulfillment.pt')
    
    entries = ()

    columns = [
        column.GetterColumn( title=_(u"Date"), getter=lambda i,f:i['creation_date'].strftime('%m/%d/%y') ),
        column.GetterColumn( title=_(u"Order Number"), getter=orderLink ),
        column.GetterColumn( title=_(u"Line Items"), getter=lambda i,f:i['line_items'] ),
        column.GetterColumn( title=_(u"Packages"), getter=lambda i,f: 1 ),
        column.GetterColumn( title=_(u"Pieces"), getter=lambda i,f: i['pieces'] )
        ]

    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}
        self.widgets = form.setUpDataWidgets(
            self.form_fields, self.prefix, self.context, self.request,
            ignore_request=ignore_request
            )

    def listing( self ):
        formatter = table.StandaloneFullFormatter(
            self.context,
            self.request,
            self.entries,
            prefix='form',
            visible_column_names = [c.name for c in self.columns],
            columns = self.columns
            )
        formatter.cssClasses['table'] = 'listing'
        return formatter()

    @form.action( _(u"Generate Report"), condition=form.haveInputWidgets )
    def generate_report( self, action, data ):
        self.entries = report.fulfillment_history( data['start_date'], data['end_date'] )
        self.summary = report.fulfillment_summary( data['start_date'], data['end_date'] )
        
                                                   
        



"""
$Id: $
"""

from Products.Five.browser import BrowserView

from zc.table import column, table

from getpaid.report import report
from getpaid.report.i18n import _

def orderLink( item, formatter):
    if item.order_id is None:
        return item.action
    oid = item.order.order_zid
    return u'<a href="@@admin-manage-order/%s/@@fulfillment">%s</a>'%( oid, oid )

class ProductHistoryReport( BrowserView ):

    columns = [
        column.GetterColumn( title=_(u"Date"), getter=lambda i,f:i.date.strftime('%m/%d/%y') ),
        column.GetterColumn( title=_(u"Order Number"), getter=orderLink ),
        column.GetterColumn( title=_(u"Quantity"), getter=lambda i,f:i.quantity ),
        column.GetterColumn( title=_(u"On Hand"), getter=lambda i,f: i.stock ),        
        ]
    
    def listing( self ):
        entries = report.product_history( self.context )
        formatter = table.StandaloneFullFormatter(
            self.context,
            self.request,
            entries,
            prefix='form',
            visible_column_names = [c.name for c in self.columns],
            columns = self.columns
            )

        formatter.cssClasses['table'] = 'listing'
        return formatter()
    

            
        
    



"""
$Id: $
"""

from Products.Five.browser import BrowserView
from zc.table import column, table
from getpaid.report import report
from getpaid.report.i18n import _


class BackorderReport( BrowserView ):

    report_name = _(u"Backordered Products")
    
    columns = [
        column.GetterColumn( title=_(u"Product Code"), getter=lambda i,f:i.product_code ),
        column.GetterColumn( title=_(u"Description"), getter=lambda i,f:i.resolve().Title() ),
        column.GetterColumn( title=_(u"Total Backordered"), getter=lambda i,f:abs(i.stock_reserve) ),
        ]

    def listing( self ):
        entries = report.backorder_products( supplier_id=None )
        formatter = table.StandaloneFullFormatter(
            self.context,
            self.request,
            entries,
            prefix='form',
            visible_column_names = [c.name for c in self.columns],
            columns = self.columns
            )
        formatter.cssClasses['table'] = 'listing'
        return formatter()
        

    



from setuptools import setup, find_packages

import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

setup(
    name="getpaid.report",
    version="0.1.1",
    author='Kapil Thangavelu',
    author_email='kapil.foss@gmail.com',
    description="""Relational Reports For getpaid.core.""",
    long_description = (
        read('README.txt')
        + '\n' +
        read('CHANGES.txt')
        + '\n' +
        'Detailed Documentation\n'
        '**********************\n'
        + '\n' +
        read('src', 'getpaid', 'report', 'readme.txt')
        + '\n' +
        'Download\n'
        '**********************\n'
        ),
    license='X11',
    keywords="zope zope3",
    url="http://code.google.com/p/getpaid",
    install_requires=['setuptools',
                      'getpaid.core',
                      'getpaid.warehouse',
                      'zope.index', # getpaid.core dep
                      'zope.app.intid', # getpaid.core dep
                      'zope.viewlet',
                      'SQLAlchemy>0.4'], # use in plone means bad requires spec
    dependency_links=['http://download.zope.org/distribution/',],
    packages=find_packages('src', exclude=["*.tests", "*.ftests"]),
    package_dir= {'':'src'},
    namespace_packages=['getpaid'],
    package_data = {
    '': ['*.txt', '*.zcml', '*.pt'],
    },
    zip_safe=False,
    entry_points="""
    [console_scripts]
    setup-report-db = getpaid.report.schema:main
    """
    )
